package com.zy.ossservice.service.impl;

import com.zy.commonutils.exception.FileException;
import com.zy.ossservice.service.FileSolveService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

/**
 * @author 张雨 - 栀
 * @version 1.0
 * @create 2022/3/19 20:12
 */
@Service
public class FileSolveServiceImpl implements FileSolveService {
    // 90M
    private static final Long PIC_MAX_SIZE = 94371840L;
    // 1G
    private static final Long MP4_MAX_SIZE = 1073741824L;

    @Value("${data.static.url}")
    private String baseUrl;

    /**
     * 保存视频文件 至本地
     * @param multipartFile 文件流
     * @return 返回 保存好的文件名
     */
    @Override
    public String saveMp4(MultipartFile multipartFile) {
        File file = getLocalDir("mp4");
        String fileName = "";
        if (multipartFile != null) {
            long size = multipartFile.getSize();
            if (size > MP4_MAX_SIZE) {throw new FileException();}
            String originalFilename = multipartFile.getOriginalFilename();
            String uuid = UUID.randomUUID().toString().replaceAll("-", "");
            fileName = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + "_" + uuid + "_" + originalFilename;
            // 组合 路径名
            String filePath = file + "\\" + fileName;
            // 保存到本地
            try {multipartFile.transferTo(new File(filePath));} catch (IOException e) { throw new FileException();}
        }else{throw new FileException();}
        return baseUrl + "mp4/" + fileName;
    }

    /**
     * 保存图片文件 至本地
     * @param multipartFile 文件流
     * @return 返回 保存好的文件名
     */
    @Override
    public String saveImgs(MultipartFile multipartFile) {
        File file = getLocalDir("img");
        String fileName = "";
        if (multipartFile != null) {
            long size = multipartFile.getSize();
            if (size > PIC_MAX_SIZE) {
                throw new FileException();
            }
            // 获取上传时的文件名
            String originalFilename = multipartFile.getOriginalFilename();
            // 文件名后缀
//            String subffix = fileType(originalFilename);
            // 文件名
//            String prefix = originalFilename.substring(0, originalFilename.lastIndexOf("."));
            // 组合 最终文件名
            String uuid = UUID.randomUUID().toString().replaceAll("-", "");
            fileName = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + "_" + uuid + "_" + originalFilename;
            // 组合 路径名
            String filePath = file + "\\" + fileName;
            try {
                multipartFile.transferTo(new File(filePath));
            } catch (IOException e) { throw new FileException();}
        }else{throw new FileException();}
        return baseUrl + "img/" + fileName;
    }

    private static String fileType(String fileName){
        return fileName.substring(fileName.lastIndexOf(".") + 1);
    }

    /**
     * 获取项目根路径 用于保存图片
     * @return 项目存储路径
     */
    private File getLocalDir(String type){
        String relativelyPath = System.getProperty("user.dir") + "\\data\\" + type;
        File file = new File(relativelyPath);
        if (!file.exists()) {
            boolean isMkdirs = file.mkdirs();
            if(!isMkdirs){ throw new FileException(); }
        }
        return file;
    }
}
